@@ -28,7 +28,10 @@
 #include <unistd.h>
 #include <setjmp.h>
 
-#include "config.h"
+#if HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include "command.h"
 #include "strlist.h"
 #include "ui.h"
@@ -41,6 +44,9 @@ extern int fdisk_partsize_part; /* ?? */
 
 /* not 0 when fdisk should print the partition table? */
 extern int fdisk_list_table;
+extern int init_disk_flags(PedDisk *disk);
+extern void fdisk_print_partition_size(PedDisk *disk);
+extern void fdisk_do_list_devices (PedDisk* disk);
 
 #if HAVE_LOCALE_H
 # include <locale.h>
@@ -53,45 +59,40 @@ extern int fdisk_list_table;
 /* BEGIN readline stuff */
 #if HAVE_LIBREADLINE
 
-/* Termcap need this buffer to initialize terminal database, I think
-   the db is stored into this buffer but I'm not sure. */
-enum { TERMCAP_BUFFER_SIZE = 2048 };
-static char termcap_buffer[TERMCAP_BUFFER_SIZE] = {0};
+/* If the macro "HAVE_LIBREADLINE corresponds to a digit other than zero
+ * the readline is present along with the respective `readline/readline.h'
+ * and `readline/history.h' header file's */
 
-/* Use this flag to know if termcap is propely initialized
-   and ready or should not be used */
-static enum { TERMCAP_OK, TERMCAP_ERROR } termcap_status;
 
 /* termcap library */
-# if HAVE_TERMCAP_H
-#  include <termcap.h>
-# else
-#  if HAVE_LIBTERMCAP
-    extern int tgetnum (char* key);
-#  endif
-# endif /* HAVE_TERMCAP_H */
+#	if HAVE_LIBTERMCAP
+	/* Termcap need this buffer to initialize terminal database, I think
+	 * the db is stored into this buffer but I'm not sure. */
+		enum { TERMCAP_BUFFER_SIZE = 2048 };
+		static char termcap_buffer[TERMCAP_BUFFER_SIZE] = {0};
 
-/* readline/readline.h */
-# if HAVE_READLINE_READLINE_H
-#  include <readline/readline.h>
-# else
-   extern char * readline ();
-   extern void add_history ();
-# endif /* HAVE_READLINE_READLINE_H */
+		/* Use this flag to know if termcap is propely initialized
+			 and ready or should not be used */
+		static enum { TERMCAP_OK, TERMCAP_ERROR } termcap_status;
+#		if HAVE_TERMCAP_H
+#			include <termcap.h>
+# 	else
+    	extern int tgetnum (char* key);
+#  	endif /* HAVE_TERMCAP_H */
+# endif /* HAVE_LIBTERMCAP */
 
-/* history library */
-# if HAVE_READLINE_HISTORY_H
-#  include <readline/history.h>
-# else
-#  define _add_history_unique add_history
-# endif
+# include <readline/readline.h>
+# include <readline/history.h>
   
-
+/* Both `HAVE_RL_COMPLETION_MATCHES' and `completion_matches' are unused */
+#if 0
 # ifndef HAVE_RL_COMPLETION_MATCHES
 #  define rl_completion_matches completion_matches
 # endif
+#endif /* 0 */
 
-# ifndef rl_compentry_func_t
+# if !HAVE_RL_COMPENTRY_FUNC_T
+	/* cast to (void*) */
 #  define rl_compentry_func_t void
 # endif
 
@@ -121,15 +122,17 @@ static struct {
   sigjmp_buf	 jmp_state;
 } readline_state;
 
-static PedExceptionOption current_exception_opt = 0;
+// unused
+//static PedExceptionOption current_exception_opt = 0;
 
 /* If we don't want the possibilities to appear on the prompt, this
    should be set to 0. TODO: Making this without a global variable
    might be a good idea */
-static prompt_possibilities = 1;
+static int prompt_possibilities = 1;
 
 
-#if HAVE_LIBREADLINE && HAVE_LIBTERMCAP
+#if HAVE_LIBREADLINE
+#	if HAVE_LIBTERMCAP
 /* This function initialize the termcap library */
 void init_termcap_library(void)
 {
@@ -166,6 +169,7 @@ void init_termcap_library(void)
  _Error:
   termcap_status = TERMCAP_ERROR;
 }
+#endif /* HAVE_LIBTERMCAP */
 
 /* returns matching commands for text */
 static char*
@@ -628,9 +632,7 @@ fdisk_interactive_mode (PedDisk** disk, FdiskCommand* cmd_list[])
 int
 fdisk_interactive_mode(PedDevice **dev, FdiskCommand* cmd_list[])
 {
-  char*	line;
   PedDisk *disk;
-  StrList* list;
   StrList* command_names = fdisk_command_get_names (cmd_list);
     
   puts (interface_name);
@@ -653,6 +655,10 @@ fdisk_interactive_mode(PedDevice **dev, FdiskCommand* cmd_list[])
 		  "content won't be recoverable.\n"));
 	  if((disk = ped_disk_new_fresh(*dev, ped_disk_type_get("msdos"))) == NULL)
 	    return EXIT_FAILURE;
+	  if(!init_disk_flags(disk)) {
+	  	ped_disk_destroy(disk);
+		return EXIT_FAILURE;
+	  }
 	}
       else
 	{
@@ -663,7 +669,8 @@ fdisk_interactive_mode(PedDevice **dev, FdiskCommand* cmd_list[])
 	  char *str;
 
 	  for(label_type = ped_disk_type_get_next(NULL);
-	      label_type = ped_disk_type_get_next(label_type); )
+	      label_type != NULL;
+				label_type = ped_disk_type_get_next(label_type) )
 	    {
 	      supp = str_list_append(supp, label_type->name);
 	      supp = str_list_append(supp, " ");
@@ -706,6 +713,11 @@ fdisk_interactive_mode(PedDevice **dev, FdiskCommand* cmd_list[])
 	  if(!disk)
 	    return EXIT_FAILURE; /* what bad? :( */
 
+	  if (!init_disk_flags(disk)) {
+	  	ped_disk_destroy(disk);
+	  	return EXIT_FAILURE;
+	  }
+	  
 	  /* Free other resources */
 	  free(str);
 	  free(supported_types);
@@ -717,6 +729,10 @@ fdisk_interactive_mode(PedDevice **dev, FdiskCommand* cmd_list[])
       disk = ped_disk_new(*dev);
       if(!disk)
 	return EXIT_FAILURE;
+      if(!init_disk_flags(disk)) {
+	ped_disk_destroy(disk);
+        return EXIT_FAILURE;
+      }
     }
   
   /***
@@ -781,8 +797,6 @@ fdisk_interactive_mode(PedDevice **dev, FdiskCommand* cmd_list[])
 int
 fdisk_interactive_menu (PedDisk** disk, FdiskCommand* cmd_list[], int menu)
 {
-	char*		line;
-	StrList*	list;
 	StrList*	command_names = fdisk_command_get_names (cmd_list);
 
 	/* FIXME yucky, nasty, evil hack */
@@ -857,7 +871,7 @@ fdisk_screen_width ()
     return 32768;	/* no wrapping ;) */
 
 /* HACK: don't specify termcap separately - it'll annoy the users. */
-#if HAVE_LIBREADLINE
+#if HAVE_LIBREADLINE && HAVE_LIBTERMCAP
   if(termcap_status == TERMCAP_OK)
     width = tgetnum ("co");
 #endif
@@ -1204,7 +1218,7 @@ fdisk_command_line_get_integer (const char* prompt, int* value)
 	return valid;
 }
 
-fdisk_command_line_get_llinteger (const char* prompt, long long* value)
+int fdisk_command_line_get_llinteger (const char* prompt, long long* value)
 {
 	char	def_str [22];
 	char*	input;
@@ -1411,15 +1425,15 @@ fdisk_command_line_get_sector (const char* prompt, PedDevice* dev, PedSector* va
 	if (input && *value && !strcmp (input, def_str)) {
 		if (range) {
 			*range = ped_geometry_new (dev, *value, 1);
-			ped_free (def_str);
+			PED_FREE (def_str);
 			return *range != NULL;
 		}
 
-		ped_free (def_str);
+		PED_FREE (def_str);
 		return 1;
 	}
 
-	ped_free (def_str);
+	PED_FREE (def_str);
 	if (!input) {
 		*value = 0;
 		if (range)
@@ -1451,7 +1465,7 @@ fdisk_init_ui ()
 
 #if HAVE_LIBREADLINE
   rl_initialize ();
-  rl_attempted_completion_function = (CPPFunction*) fdisk_complete_function;
+	rl_attempted_completion_function = (CPPFunction*) fdisk_complete_function;
 #endif /* HAVE_LIBREADLINE */
 
   readline_state.in_readline = 0; 
@@ -1493,7 +1507,7 @@ fdisk_command_line_get_part_flag (const char* prompt, const PedPartition* part,
 
 	if (flag_name) {
 		*flag = ped_partition_flag_get_by_name (flag_name);
-		ped_free (flag_name);
+		PED_FREE (flag_name);
 		return 1;
 	} else {
 		return 0;
